#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/5/18 9:56 下午
# @Author  : shuming.wsm
# @Site    : 
# @File    : up.py
# @Software: PyCharm
# @Desc    : 上行包
import json

from agent.packages.base import PackageType
from agent.tools import build_msg_id


class ACK(object):
    agentClient = None
    message = None
    msgId = None
    data = None

    def __init__(self,  agent_client):
        self.msgId = build_msg_id()
        self.agentClient = agent_client
        self.message = {"msgId": self.msgId}
        self.data = "%d%s" % (PackageType.ACK.value, json.dumps(self.message))

    def process(self):
        # 构造ACK发送
        self.agentClient.send_message(self.data)


class PING(object):
    agentClient = None
    data = None

    def __init__(self, agent_client):
        self.data = "%d" % PackageType.PING.value
        self.agentClient = agent_client

    def process(self):
        self.agentClient.send_message(self.data)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/5/19 10:24 下午
# @Author  : shuming.wsm
# @Site    : 
# @File    : finishWork.py
# @Software: PyCharm
# @Desc    :

from agent.logger import logger
from agent.packages.message.base import Message
from agent.packages.message.send.finishWorkFail import FinishWorkFail
from agent.packages.message.send.finishWorkSuccess import FinishWorkSuccess


class FinishWork(Message):
    # 自己实现自己的finish work做好资源回收

    def do_finish_work(self):
        logger.info("do finish work")
        return True

    def finish_work_success(self):
        finish_work_success_ack = FinishWorkSuccess(self.agentClient)
        if finish_work_success_ack.is_valid():
            finish_work_success_ack.process()

    def finish_work_fail(self):
        finish_work_fail_ack = FinishWorkFail(self.agentClient)
        if finish_work_fail_ack.is_valid():
            finish_work_fail_ack.process()

    def process(self):
        try:
            if self.do_finish_work():
                logger.info("do finish work success")
                self.finish_work_success()
            else:
                logger.error("do finish work fail")
                self.finish_work_fail()
        except Exception as e:
            logger.exception("do finish work exception:%s" % e.__str__())






#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/11/8 9:52 下午
# @Author  : shuming.wsm
# @Site    : 
# @File    : package.py
# @Software: PyCharm
# @Desc    :
from enum import Enum
import json
from agent.logger import logger

from agent.packages.message.base import Message

class PackageType(Enum):
    # 链接成功， agent下行
    OPEN = 1

    # 关闭链接， agent下行
    CLOSE = 2

    # 心跳， container上行
    PING = 3

    # 心跳回执
    PONG = 4

    # 业务会话消息
    MESSAGE = 5

    # 6业务会话消息回执
    ACK = 6


class Package(object):
    packageType = None
    data = None
    message = None
    valid = False
    agentClient = None

    def __init__(self, data, agent_client):
        self.data = data
        self.agentClient = agent_client

    def is_valid(self):
        try:
            cmd = int(self.data[0])
        except Exception as e:
            raise Exception("agent package cmd error, %s" % e.__str__())
        if cmd not in (PackageType._value2member_map_):
            raise Exception("agent not stand by package type:%d" % cmd)
        self.packageType = PackageType(cmd)
        if self.packageType == PackageType.OPEN:
            logger.info("receive OPEN package")
        elif self.packageType == PackageType.PONG:
            logger.info("receive PONG package")
        elif self.packageType == PackageType.ACK:
            logger.info("receive ACK package, package:%s" % self.data)
        elif self.packageType == PackageType.MESSAGE:
            messageString = self.data[1:]
            messageData = json.loads(messageString)
            self.message = Message(messageData['header'], messageData['body'], self.agentClient)
            self.message.is_valid()
        else:
            raise Exception("not standby package type, %d" % self.packageType)
        self.valid = True
        return True

    def get_message(self):
        if self.valid:
            return self.message
        else:
            raise Exception("data not valid, please call 'is_valid'")

    def process(self):
        try:
            if self.packageType == PackageType.MESSAGE:
                if self.message.get_message_type() == "text":
                    self.message.process()
                    if self.agentClient.message_process_callback:
                        self.agentClient.message_process_callback(self.message.get_content())
                else:
                    cmd_code = self.message.get_cmd_code()
                    class_name = self.agentClient.get_cmd_code_message_class(cmd_code)
                    if not class_name:
                        self.agentClient.message_process_callback(self.message.get_content())
                    else:
                        self.message.__class__ = class_name
                    self.message.process()
            else:
                pass
        except Exception as e:
            logger.error("process package error %s, package:%s" % (e.__str__(), self.message.get_package_string()))
















#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/5/19 9:43 下午
# @Author  : shuming.wsm
# @Site    : 
# @File    : startWork.py
# @Software: PyCharm
# @Desc    :
from agent.logger import logger
from agent.packages.message.base import Message
from agent.packages.message.send.startWorkFail import StartWorkFail
from agent.packages.message.send.startWorkSuccess import StartWorkSuccess


class StartWork(Message):
    # 各个容器收到startWork不一样， 默认返回START_WORK_SUCCESS_ACK

    def do_start_work(self):
        logger.info("do start work")
        return True

    def start_work_success(self):
        start_work_success_ack = StartWorkSuccess(self.agentClient)
        if start_work_success_ack.is_valid():
            start_work_success_ack.process()

    def start_work_fail(self):
        start_work_fail_ack = StartWorkFail(self.agentClient)
        if start_work_fail_ack.is_valid():
            start_work_fail_ack.process()

    def process(self):
        try:
            if self.do_start_work():
                logger.error("start work success")
                self.start_work_success()
            else:
                logger.error("start work fail")
                self.start_work_fail()
        except Exception as e:
            logger.exception("start work exception: %s", e.__str__())
            self.start_work_fail()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/5/18 9:56 下午
# @Author  : shuming.wsm
# @Site    : 
# @File    : down.py
# @Software: PyCharm
# @Desc    : 下行包

from agent.packages.base import Package
from agent.packages.base import PackageType
from agent.logger import logger


class ACK(Package):

    def is_valid(self):
        if self.packageType == PackageType.ACK:
            return True
        else:
            raise Exception("ack cmd error, must %d" % PackageType.ACK)

    def process(self):
        logger.info("receive ack: %s" % self.data)


class PONG(Package):

    def is_valid(self):
        if self.packageType == PackageType.PONG:
            return True
        else:
            raise Exception("PONG cmd error, must %d" % PackageType.PONG)

    def process(self):
        pass


class Open(Package):

    def is_valid(self):
        if self.packageType == PackageType.OPEN:
            return True
        else:
            raise Exception("OPEN cmd error, must %d" % PackageType.Open.valueo)

    def process(self):
        pass




import asyncio
import time

import websockets
from flask import Flask, request
import json
import _thread
app = Flask(__name__)

cmd_queue = asyncio.Queue()
text_queue = asyncio.Queue()


async def ws_send_cmd(websocket):
    while True:
        try:
            if not cmd_queue.empty():
                send_cmd = await cmd_queue.get()
                print("> send cmd: ", send_cmd)
                await websocket.send("5" + json.dumps(send_cmd))
            else:
                await asyncio.sleep(0.1)
                continue
        except Exception as e:
            await asyncio.sleep(0.1)
            print("get cmd queue error", e.__str__())


async def ws_send_text(websocket):
    while True:
        try:
            if not text_queue.empty():
                send_text = await text_queue.get()
                print("> send text: ", send_text)
                await websocket.send("5" + json.dumps(send_text))
            else:
                await asyncio.sleep(0.1)
                continue
        except Exception as e:
            await asyncio.sleep(0.1)
            print("get text queue error", e.__str__())


async def ws_recv_handler(websocket):
    while True:
        try:
            recv_msg = await asyncio.wait_for(websocket.recv(), timeout=0.1)
            if not recv_msg or recv_msg == '':
                await  asyncio.sleep(0.1)
                continue
            print("< receive_msg: ", recv_msg)
            package_cmd = recv_msg[0]
            if package_cmd == '3':
                print("> send_msg: ", '4')
                await  websocket.send('4')
            if package_cmd == '5':
                package = json.load(package_cmd[1:])
                ack = {
                   'msgId': package['header']['msgId']
                }
                print("> send_msg: ", '6'+json.dumps(ack))
                await websocket.send('6' + json.dumps(ack))
            if package_cmd == '6':
                pass
        except Exception as e:
            print(e.__str__())


async def serve(websocket, path):
    await asyncio.gather(
        ws_send_cmd(websocket),
        ws_send_text(websocket),
        ws_recv_handler(websocket)
    )


def start_ws_server(addr, port):
    event_loop = asyncio.new_event_loop()
    asyncio.set_event_loop(event_loop)
    start_server = websockets.serve(serve, addr, port)
    event_loop.run_until_complete(start_server)
    asyncio.get_event_loop().run_forever()


@app.route('/cmd/send', methods=['POST'])
def send_cmd():
    data = json.loads(request.data.decode('utf-8'))
    cmd_queue.put_nowait(data)
    resp = {'cmd': data,
            'success': True}
    return json.dumps(resp)


@app.route('/text/send', methods=['POST'])
def send_text():
    data = json.loads(request.data.decode('utf-8'))
    text_queue.put_nowait(data)
    resp = {'text': data, 'success': True}
    return json.dumps(resp)


def start_flask():
    app.run(host='0.0.0.0', port=8901)


if __name__ == "__main__":
    _thread.start_new_thread(start_flask, ())
    start_ws_server('localhost', 8101)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/11/5 4:10 下午
# @Author  : shuming.wsm
# @Site    : 
# @File    : logger.py
# @Software: PyCharm
# @Desc    :
import logging
import platform

logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(name)s - %(levelname)s - %(filename)s:%(funcName)s:%(lineno)s |%(message)s|')
logger = logging.getLogger("agent")
if platform.system() == 'Linux':
    f_handler = logging.FileHandler('~/logs/agent.log', encoding='UTF-8')
else:
    f_handler = logging.FileHandler('/tmp/agent.log', encoding='UTF-8')
    
c_handler = logging.StreamHandler()
c_handler.setLevel(logging.DEBUG)
c_format = logging.Formatter('%(name)s - %(levelname)s - %(message)s')
c_handler.setFormatter(c_format)
logger.addHandler(c_handler)
logger.addHandler(f_handler)
